@@ -29,7 +29,6 @@
 #include <QQuickItem>
 #include <QQuickWindow>
 #include <QTimer>
-#include <QVersionNumber>
 
 #include <KFileItemActions>
 #include <KFileItemListProperties>
@@ -108,6 +107,11 @@ QString Thumbnailer::iconName() const
     return m_iconName;
 }
 
+bool Thumbnailer::menuVisible() const
+{
+    return m_menuVisible;
+}
+
 void Thumbnailer::showContextMenu(int x, int y, const QString &path, QQuickItem *ctx)
 {
     if (!ctx || !ctx->window()) {
@@ -124,6 +128,11 @@ void Thumbnailer::showContextMenu(int x, int y, const QString &path, QQuickItem
     QMenu *menu = new QMenu();
     menu->setAttribute(Qt::WA_DeleteOnClose, true);
 
+    connect(menu, &QMenu::aboutToHide, this, [this] {
+        m_menuVisible = false;
+        emit menuVisibleChanged();
+    });
+
     if (KProtocolManager::supportsListing(url)) {
         QAction *openContainingFolderAction = menu->addAction(QIcon::fromTheme("folder-open"), i18n("Open Containing Folder"));
         connect(openContainingFolderAction, &QAction::triggered, [url] {
@@ -170,19 +179,27 @@ void Thumbnailer::showContextMenu(int x, int y, const QString &path, QQuickItem
         }
     };
 
-    //pre 5.8.0 QQuickWindow code is "item->grabMouse(); sendEvent(item, mouseEvent)"
-    //post 5.8.0 QQuickWindow code is sendEvent(item, mouseEvent); item->grabMouse()
-    if (QVersionNumber::fromString(qVersion()) > QVersionNumber(5, 8, 0)) {
-        QTimer::singleShot(0, ctx, ungrabMouseHack);
-    }
-    else {
-        ungrabMouseHack();
-    }
+    QTimer::singleShot(0, ctx, ungrabMouseHack);
     //end workaround
 
+    QPoint pos;
+    if (x == -1 && y == -1) { // align "bottom left of ctx"
+        menu->adjustSize();
+
+        pos = ctx->mapToGlobal(QPointF(0, ctx->height())).toPoint();
+
+        if (!qApp->isRightToLeft()) {
+            pos.rx() += ctx->width();
+            pos.rx() -= menu->width();
+        }
+    } else {
+        pos = ctx->mapToGlobal(QPointF(x, y)).toPoint();
+    }
 
-    const QPoint pos = ctx->mapToGlobal(QPointF(x, y)).toPoint();
     menu->popup(pos);
+
+    m_menuVisible = true;
+    emit menuVisibleChanged();
 }
 
 void Thumbnailer::generatePreview()
