@@ -21,7 +21,6 @@
 #include <QAction>
 #include <QGuiApplication>
 #include <QDebug>
-#include <QDesktopWidget>
 #include <QQuickItem>
 #include <QQmlContext>
 #include <QScreen>
@@ -29,6 +28,7 @@
 #include <QClipboard>
 #include <QPlatformSurfaceEvent>
 
+#include <KAuthorized>
 #include <KWindowSystem>
 #include <KWindowEffects>
 #include <KAuthorized>
@@ -99,7 +99,7 @@ View::View(QWindow *)
         package.setPath(packageName);
     }
 
-    m_qmlObj->setSource(QUrl::fromLocalFile(package.filePath("runcommandmainscript")));
+    m_qmlObj->setSource(package.fileUrl("runcommandmainscript"));
     m_qmlObj->engine()->rootContext()->setContextProperty(QStringLiteral("runnerWindow"), this);
     m_qmlObj->completeInitialization();
 
@@ -222,9 +222,7 @@ bool View::event(QEvent *event)
     const bool retval = Dialog::event(event);
     bool setState = event->type() == QEvent::Show;
     if (event->type() == QEvent::PlatformSurface) {
-        if (auto e = dynamic_cast<QPlatformSurfaceEvent*>(event)) {
-            setState = e->surfaceEventType() == QPlatformSurfaceEvent::SurfaceCreated;
-        }
+        setState = (static_cast<QPlatformSurfaceEvent*>(event)->surfaceEventType() == QPlatformSurfaceEvent::SurfaceCreated);
     }
     if (setState) {
         KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager);
@@ -403,6 +401,11 @@ void View::displayConfiguration()
     QProcess::startDetached(QStringLiteral("kcmshell5"), QStringList() << QStringLiteral("plasmasearch"));
 }
 
+bool View::canConfigure() const
+{
+    return KAuthorized::authorizeControlModule(QStringLiteral("kcm_plasmasearch.desktop"));
+}
+
 QStringList View::history() const
 {
     return m_history;
