@@ -409,12 +409,11 @@ void KSMServer::autoStart2()
     QDBusInterface kded( QStringLiteral( "org.kde.kded5" ),
                          QStringLiteral( "/kded" ),
                          QStringLiteral( "org.kde.kded5" ) );
-    kded.call( QStringLiteral( "loadSecondPhase" ) );
-
-#ifdef KSMSERVER_STARTUP_DEBUG1
-    qCDebug(KSMSERVER)<< "kded" << t.elapsed();
-#endif
+    auto pending = kded.asyncCall( QStringLiteral( "loadSecondPhase" ) );
 
+    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pending, this);
+    QObject::connect(watcher, &QDBusPendingCallWatcher::finished, this, &KSMServer::secondKDEDPhaseLoaded);
+    QObject::connect(watcher, &QDBusPendingCallWatcher::finished, watcher, &QObject::deleteLater);
     runUserAutostart();
 
     if (kcminitSignals) {
@@ -427,6 +426,15 @@ void KSMServer::autoStart2()
     } else {
         QTimer::singleShot(0, this, &KSMServer::kcmPhase2Done);
     }
+}
+
+void KSMServer::secondKDEDPhaseLoaded()
+{
+
+#ifdef KSMSERVER_STARTUP_DEBUG1
+    qCDebug(KSMSERVER)<< "kded" << t.elapsed();
+#endif
+
     if( !defaultSession())
         restoreLegacySession(KSharedConfig::openConfig().data());
 
@@ -677,20 +685,15 @@ void KSMServer::slotAutoStart()
             return;
         }
         KService service(serviceName);
-        qCInfo(KSMSERVER) << "Starting autostart service " << serviceName;
-        auto p = new KProcess(this);
         auto arguments = KIO::DesktopExecParser(service, QList<QUrl>()).resultingArguments();
         if (arguments.isEmpty()) {
-            qCInfo(KSMSERVER) << "failed to parse" << serviceName << "for autostart";
+            qCWarning(KSMSERVER) << "failed to parse" << serviceName << "for autostart";
             continue;
         }
+        qCInfo(KSMSERVER) << "Starting autostart service " << serviceName << arguments;
         auto program = arguments.takeFirst();
-        p->setProgram(program, arguments);
-        p->start();
-        connect(p, static_cast<void (QProcess::*)(int)>(&QProcess::finished), [p](int exitCode) {
-            qCInfo(KSMSERVER) << "autostart service" << p->program() << "finished with exit code " << exitCode;
-            p->deleteLater();
-        });
+        if (!QProcess::startDetached(program, arguments))
+            qCWarning(KSMSERVER) << "could not start" << serviceName << ":" << program << arguments;
     } while (true);
     // Loop till we find a service that we can start.
 }
