@@ -431,11 +431,8 @@ void DBusMenuImporter::slotGetLayoutFinished(QDBusPendingCallWatcher *watcher)
                 sendClickedEvent(id);
             });
 
-            if (action->menu()) {
-                auto menu = action->menu();
-                connect(menu, &QMenu::aboutToShow, this, [menu, this]() {
-                   updateMenu(menu);
-                });
+            if (QMenu *menuAction = action->menu()) {
+                connect(menuAction, &QMenu::aboutToShow, this, &DBusMenuImporter::slotMenuAboutToShow, Qt::UniqueConnection);
             }
             connect(menu, &QMenu::aboutToHide, this, &DBusMenuImporter::slotMenuAboutToHide, Qt::UniqueConnection);
 
@@ -480,6 +477,9 @@ void DBusMenuImporter::updateMenu(QMenu * menu)
     watcher->setProperty(DBUSMENU_PROPERTY_ID, id);
     connect(watcher, &QDBusPendingCallWatcher::finished, this,
         &DBusMenuImporter::slotAboutToShowDBusCallFinished);
+
+    // Firefox deliberately ignores "aboutToShow" whereas Qt ignores" opened", so we'll just send both all the time...
+    d->sendEvent(id, QStringLiteral("opened"));
 }
 
 void DBusMenuImporter::slotAboutToShowDBusCallFinished(QDBusPendingCallWatcher *watcher)
@@ -527,11 +527,7 @@ void DBusMenuImporter::slotMenuAboutToShow()
     QMenu *menu = qobject_cast<QMenu*>(sender());
     Q_ASSERT(menu);
 
-    QAction *action = menu->menuAction();
-    Q_ASSERT(action);
-
-    int id = action->property(DBUSMENU_PROPERTY_ID).toInt();
-    d->sendEvent(id, QStringLiteral("opened"));
+    updateMenu(menu);
 }
 
 QMenu *DBusMenuImporter::createMenu(QWidget *parent)
