@@ -132,6 +132,8 @@ bool CurrentContainmentActionsModel::append(const QString &action, const QString
     m_removedTriggers.removeAll(action);
 
     appendRow(item);
+
+    emit configurationChanged();
     return true;
 }
 
@@ -166,6 +168,8 @@ void CurrentContainmentActionsModel::update(int row, const QString &action, cons
             m_plugins[action]->restore(tempConfig);
             setData(idx, m_plugins[action]->pluginInfo().property(QStringLiteral("X-Plasma-HasConfigurationInterface")).toBool(), HasConfigurationInterfaceRole);
         }
+
+        emit configurationChanged();
     }
 }
 
@@ -178,10 +182,11 @@ void CurrentContainmentActionsModel::remove(int row)
         delete m_plugins[action];
         m_plugins.remove(action);
         m_removedTriggers << action;
+        emit configurationChanged();
     }
 }
 
-void CurrentContainmentActionsModel::showConfiguration(int row)
+void CurrentContainmentActionsModel::showConfiguration(int row, QQuickItem *ctx)
 {
     const QString action = itemData(index(row, 0)).value(ActionRole).toString();
 
@@ -193,7 +198,11 @@ void CurrentContainmentActionsModel::showConfiguration(int row)
     configDlg->setAttribute(Qt::WA_DeleteOnClose);
     QLayout *lay = new QVBoxLayout(configDlg);
     configDlg->setLayout(lay);
-    configDlg->setWindowModality(Qt::WindowModal);
+    if (ctx && ctx->window()) {
+        configDlg->setWindowModality(Qt::WindowModal);
+        configDlg->winId(); // so it creates the windowHandle();
+        configDlg->windowHandle()->setTransientParent(ctx->window());
+    }
 
     Plasma::ContainmentActions *pluginInstance = m_plugins[action];
     //put the config in the dialog
@@ -218,6 +227,8 @@ void CurrentContainmentActionsModel::showConfiguration(int row)
                 pluginInstance->configurationAccepted();
             });
 
+    connect(configDlg, &QDialog::accepted, this, &CurrentContainmentActionsModel::configurationChanged);
+
     connect(pluginInstance, &QObject::destroyed, configDlg, &QDialog::reject);
 
     configDlg->show();
